#include "xfiledlg.h"
#include "xwindow.h"

#include <stdlib.h>


/*@ 
@class XFileDialog
@parent XSystemDialog
@type overview
@symbol _
*/


/*@ XFileDialog :: XFileDialog(const XWindow * owner, const char *f, const char *title, const char *fileType, const LONG options)
@group constructors/destructors
@remarks Open the system defined file-dialog
@parameters <t '' c=2>
				XWindow * theOwner           owner-window
            char * fileMask              the file-mask to be displayed (default is <*.*>)
            char * theTitle              title of the dialog (default is NULL)
            char * filetype              file-type to be displayed (default is NULL)
            LONG options                 options, valid values are:
														<t '' c=2>
                                             FD_OPEN         open-dialog
                                             FD_CENTER       dialog is centered
                                             FD_SAVEAS_      saveas-dialog
                                             FD_MULTIPLESEL  multiple file-selection
                                             FD_HELP         the dialog has a help-button. To display help the owner-window must be associated with a help-instance and in th help-file there must be an entry with an id of 267 (res=267)
                                             FD_MODELESS     show modeless
                                             FD_APPLYBUTTON  apply-button
                                             FD_EAS          use EAs
                                             FD_FILTERUNION  use filter from string-filter and EAs
															FD_HELPBUTTON	 show a help-button
														</t>
                                         (default is FD_OPEN | FD_CENTER, can be or-ed)
				</t>
*/
XFileDialog :: XFileDialog(const XWindow * owner, const char *f, const char *title, const char *fileType, const LONG options)
{
	FILEDLG fild;

	fileName = NULL;
	memset(&fild, 0, sizeof(FILEDLG));
	fild.cbSize = sizeof(FILEDLG);
	fild.fl = options;
	fild.pszTitle = (PSZ) title;
	fild.pszIType = (PSZ) fileType;


	strcpy(fild.szFullFile, f);

	WinFileDlg(HWND_DESKTOP, owner->GetHandle(), &fild);
	pap = fild.papszFQFilename;
	numbers = fild.ulFQFCount;
	command = fild.lReturn;
	fileName = (char *) malloc(strlen(fild.szFullFile) + 1);
	strcpy(fileName, fild.szFullFile);
}


/*@ XFileDialog::GetFileName()
@group misc
@remarks If the file-dialog is proceeded you can ask for the
selected filename. If FD_MULTIPLESEL is specified, at first ask with GetFileCount()
for the number of files selected.
@parameters XString * buffer                the buffer which will contain the filename
            SHORT index                     zero-based index of filename to get
                                            (default is 0)
*/



XFileDialog :: ~XFileDialog()
{
	WinFreeFileDlgList(pap);
	free(fileName);
}

